package net.sf.jguard.jsf.authentication.callbacks;

import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.LanguageCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AuthenticationUtils;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.callbacks.CallbackHandlerUtils;
import net.sf.jguard.ext.authentication.callbacks.JCaptchaCallback;
import net.sf.jguard.jee.authentication.http.HttpConstants;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.octo.captcha.service.CaptchaService;

public class PortletCallbackHandler implements CallbackHandler {

	private PortletRequest portletRequest;
	private PortletResponse portletResponse;
	private boolean afterRegistration;
	private static final Logger logger = LoggerFactory.getLogger(PortletCallbackHandler.class);
	private static String authSchemes=HttpConstants.FORM_AUTH;
	private static String loginField="login";
	private static String passwordField="password";
	
	/**
	 * constructor required by javadoc of the CallbackHandler interface.
	 */
	public PortletCallbackHandler(){
		super();
	}
	
	/**
	 * constructor.
	 * @param request
	 * @param response
	 * @param authSchemes
	 */
	public PortletCallbackHandler(PortletRequest request,PortletResponse response){
		this.portletRequest = request;
		afterRegistration = ((Boolean)request.getAttribute(CoreConstants.REGISTRATION_DONE)).booleanValue();
		this.portletResponse = response;
	}
	
	public void handle(Callback[] callbacks) throws IOException,
			UnsupportedCallbackException {
		boolean httpRelatedAuthScheme = false;
        for (int i = 0; i< callbacks.length;i++){
            Callback callback = callbacks[i];
            if (callback  instanceof LanguageCallback) {
				LanguageCallback languageCallback = (LanguageCallback)callback;
				Locale locale = portletRequest.getLocale();
				languageCallback.setLocale(locale);
			}
        }
        
        //authentication schemes part
        logger.debug("authSchemes="+authSchemes);
        String[] schemes = authSchemes.split(",");
        List authSchemesList = Arrays.asList(schemes);
        Iterator itAutSchemes = authSchemesList.iterator();
        while(itAutSchemes.hasNext()){
                String scheme = (String)itAutSchemes.next();
                //FORM, BASIC, and DIGEST are mutual exclusive
                if(!httpRelatedAuthScheme && HttpConstants.FORM_AUTH.equalsIgnoreCase(scheme)){
                        grabFormCredentials(this.portletRequest,callbacks);
                        httpRelatedAuthScheme = true;
                }
                //CLIENT_CERT can be used with another authentication mechanism
                //defined above
                if(HttpConstants.CLIENT_CERT_AUTH.equalsIgnoreCase(scheme)){
                        boolean certificatesFound = grabClientCertCredentials(this.portletRequest,callbacks);
                        if(!certificatesFound){
                                logger.info(" X509 certificates are not found ");
                        }
                }
        }

	}

	public static String getAuthSchemes() {
		return authSchemes;
	}

	public static void setAuthSchemes(String authSchemes) {
		PortletCallbackHandler.authSchemes = authSchemes;
	}

	/**
     * grab user credentials from request in the 'form' authentication metod.
     * @param request request send by the client
     * @param callbacks
     * @return authentication result : <b>true</b> when authentication succeed,<b>false</b> when authentication fails.
     */
	private  boolean  grabFormCredentials(PortletRequest request,Callback[] callbacks){
		boolean result = false;
        PortletSession session = request.getPortletSession(true);

		for(int i=0;i<callbacks.length;i++){
        	if(callbacks[i] instanceof NameCallback){
        		NameCallback nc = (NameCallback)callbacks[i];
        		String login =portletRequest.getParameter(loginField);
        		nc.setName(login);
        	}else if(callbacks[i] instanceof PasswordCallback){
        		PasswordCallback pc = (PasswordCallback)callbacks[i];
        		String strPwd = portletRequest.getParameter(passwordField);
                 if(strPwd!= null &&strPwd!=""){
        		  pc.setPassword(strPwd.toCharArray());
                 }else{
                  pc.setPassword(null);
                 }
        	}else if(callbacks[i] instanceof JCaptchaCallback){
        		JCaptchaCallback pc = (JCaptchaCallback)callbacks[i];
        		pc.setCaptchaAnswer(portletRequest.getParameter(SecurityConstants.CAPTCHA_ANSWER));
        		pc.setCaptchaService((CaptchaService)session.getPortletContext().getAttribute(SecurityConstants.CAPTCHA_SERVICE));
        		Subject subject = ((AuthenticationUtils)session.getAttribute(CoreConstants.AUTHN_UTILS)).getSubject();
        		if(subject==null ||afterRegistration){
        		    pc.setSkipJCaptchaChallenge(true);
        		}

        		pc.setSessionID(session.getId());
        	}
        }
        result = true;


		return result;
	}

	
	/**
	 * grab user credentials from request in the 'clientCert' authentication metod.
	 * @param request
	 * @param callbacks
	 * @return <code>true</code> if successfull, <code>false</code> otherwise
	 */
	private boolean grabClientCertCredentials(PortletRequest request,Callback[] callbacks) {
		if(!request.isSecure()){
			logger.warn(" certificate-based authentication MUST be do in secure mode ");
			logger.warn(" but connection is do with the non secured protocol "+request.getScheme());
			return false;
		}

		Object[] objects = (Object[]) request.getAttribute(CallbackHandlerUtils.JAVAX_SERVLET_REQUEST_X509CERTIFICATE);
		
		return CallbackHandlerUtils.grabClientCertCredentials(callbacks, objects);
	}
	
	public static String getLoginField() {
		return loginField;
	}

	public static void setLoginField(String loginField) {
		PortletCallbackHandler.loginField = loginField;
	}

	public static String getPasswordField() {
		return passwordField;
	}

	public static void setPasswordField(String passwordField) {
		PortletCallbackHandler.passwordField = passwordField;
	}
	
}
